﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Xml;

namespace SubWayDemo
{
    /// <summary>
    /// MainWindow.xaml 的交互逻辑
    /// </summary>
    public partial class MainWindow : Window
    {
        public MainWindow()
        {
            InitializeComponent();
            ReadXML();
            ShowSubWay();
        }
        string xmlPath = AppDomain.CurrentDomain.BaseDirectory + "sw.xml";
        XmlDocument doc = new XmlDocument();
        private List<DrawInfo> drInList = new List<DrawInfo>();

        public List<DrawInfo> DrInList
        {
            get
            {
                return drInList;
            }

            set
            {
                drInList = value;
            }
        }
        private void ReadXML()
        {
            doc.Load(xmlPath);
            XmlNodeList xmlNodeList = doc.DocumentElement.GetElementsByTagName("g");
            foreach (XmlNode xn in xmlNodeList)
            {
                foreach (XmlNode item in xn.ChildNodes)
                {
                    DrawInfo drawInfo = null;
                    if (item.Name == "path")
                    {
                        drawInfo = new DrawInfo()
                        {
                            Type = "path",
                            PathData = item.Attributes["d"].InnerText,
                            ForColor = item.Attributes["stroke"].InnerText,
                            Tooltip = item.Attributes["lb"].InnerText,
                        };
                    }
                    if (item.Name == "text")
                    {
                        drawInfo = new DrawInfo()
                        {
                            Type = "text",
                            ForColor = item.Attributes["fill"].InnerText,
                            X = double.Parse(item.Attributes["x"].InnerText),
                            Y = double.Parse(item.Attributes["y"].InnerText),
                            Tooltip = item.InnerText,
                            FontWeight = item.Attributes["font-weight"].InnerText
                        };
                    }
                    if (item.Name == "ellipse")
                    {
                        drawInfo = new DrawInfo()
                        {
                            Type = "ellipse",
                            ForColor = item.Attributes["stroke"].InnerText,
                            X = double.Parse(item.Attributes["cx"].InnerText) - 6.5,
                            Y = double.Parse(item.Attributes["cy"].InnerText) - 6.5,
                            Tooltip=item.Attributes["name"].InnerText,
                        };
                    }
                    if (item.Name == "image")
                    {
                        drawInfo = new DrawInfo()
                        {
                            Type = "image",
                            X = double.Parse(item.Attributes["x"].InnerText),
                            Y = double.Parse(item.Attributes["y"].InnerText),
                            Id = item.Attributes["id"].InnerText,
                            Tooltip=item.Attributes["name"].InnerText
                        };
                    }
                    if (drawInfo != null)
                        DrInList.Add(drawInfo);
                }
            }
        }
        BrushConverter brushConverter = new BrushConverter();
        private void ShowSubWay()
        {
            foreach (DrawInfo item in DrInList)
            {
                if (item.Type == "path")
                {
                    Style style = win.FindResource("pathStyle") as Style;
                    Path path = new Path()
                    {
                        Style = style,
                        Data = Geometry.Parse(item.PathData),
                        Stroke = (Brush)brushConverter.ConvertFromString(item.ForColor),
                        ToolTip = item.Tooltip
                    };
                    path.MouseMove += Path_MouseMove;
                    path.MouseLeave += Path_MouseLeave;
                    grid.Children.Add(path);
                }
                if (item.Type == "text")
                {
                    Style style = win.FindResource("textblockStyle") as Style;
                    TextBlock textBlock = new TextBlock()
                    {
                        Style = style,
                        Foreground = (Brush)brushConverter.ConvertFromString(item.ForColor),
                        Margin = new Thickness() { Left = item.X, Top = item.Y },
                        Text = item.Tooltip,
                        FontWeight = (FontWeight)new FontWeightConverter().ConvertFromString(item.FontWeight),
                    };
                    grid.Children.Add(textBlock);
                }
                if (item.Type == "ellipse")
                {
                    Style style = win.FindResource("ellipseStyle") as Style;
                    Ellipse ellipse = new Ellipse()
                    {
                        Style = style,
                        Stroke = (Brush)brushConverter.ConvertFromString(item.ForColor),
                        Margin = new Thickness() { Left = item.X, Top = item.Y },
                        ToolTip=item.Tooltip
                    };
                    ellipse.MouseMove += Ellipse_MouseMove;
                    ellipse.MouseLeave += Ellipse_MouseLeave;
                    grid.Children.Add(ellipse);
                }
                if (item.Type == "image")
                {
                    Style style = null;
                    if (item.Id == "svgjsImage12298" || item.Id == "svgjsImage12302")
                        style = win.FindResource("imgAir") as Style;
                    else
                        style = win.FindResource("imgTran") as Style;
                    Image img = new Image()
                    {
                        Style = style,
                        Margin = new Thickness() { Left = item.X, Top = item.Y },
                        ToolTip=item.Tooltip
                    };
                    img.MouseMove += Img_MouseMove;
                    img.MouseLeave += Img_MouseLeave;
                    grid.Children.Add(img);
                }
            }
        }

        private void Img_MouseLeave(object sender, MouseEventArgs e)
        {
            popEllipse.IsOpen = false;
        }

        private void Img_MouseMove(object sender, MouseEventArgs e)
        {
            Image img = (Image)sender;
            tbEllipse.Text = img.ToolTip.ToString();

            Point p = e.GetPosition(mainGrid);
            popEllipse.IsOpen = true;
            popEllipse.HorizontalOffset = p.X - 60;
            popEllipse.VerticalOffset = p.Y - mainGrid.ActualHeight - 40;
        }

        private void Ellipse_MouseLeave(object sender, MouseEventArgs e)
        {
            popEllipse.IsOpen = false;
        }

        private void Ellipse_MouseMove(object sender, MouseEventArgs e)
        {
            Ellipse ell = (Ellipse)sender;
            tbEllipse.Text = ell.ToolTip.ToString();

            Point p = e.GetPosition(mainGrid);
            popEllipse.IsOpen = true;
            popEllipse.HorizontalOffset = p.X - 60;
            popEllipse.VerticalOffset = p.Y - mainGrid.ActualHeight - 40;
        }

        private void Path_MouseMove(object sender, MouseEventArgs e)
        {
            Path path = (Path)sender;
            grpopPath.Background = path.Stroke;
            tbPath.Text = path.ToolTip.ToString();


            Point p = e.GetPosition(mainGrid);
            popPath.IsOpen = true;
            popPath.HorizontalOffset = p.X - 60;
            popPath.VerticalOffset = p.Y - mainGrid.ActualHeight - 40;
        }

        private void Path_MouseLeave(object sender, MouseEventArgs e)
        {
            popPath.IsOpen = false;
        }
        private bool mouseDown;
        private Point mouseXY;
        private double min = 1, max = 5;
        private void ContentControl_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            var con = sender as ContentControl;
            if (con == null)
            {
                return;
            }
            con.CaptureMouse();
            mouseDown = true;
            mouseXY = e.GetPosition(con);
        }

        private void ContentControl_MouseLeftButtonUp(object sender, MouseButtonEventArgs e)
        {
            var con = sender as ContentControl;
            if (con == null)
            {
                return;
            }
            con.ReleaseMouseCapture();
            mouseDown = false;
        }

        private void ContentControl_MouseMove(object sender, MouseEventArgs e)
        {
            var con = sender as ContentControl;
            if (con == null)
            {
                return;
            }
            if (mouseDown)
            {
                Domousemove(con, e);
            }
        }

        private void ContentControl_MouseWheel(object sender, MouseWheelEventArgs e)
        {
            var con = sender as ContentControl;
            if (con == null)
            {
                return;
            }
            var point = e.GetPosition(con);
            var group = grid.FindResource("tfg") as TransformGroup;
            var delta = e.Delta * 0.001;
            DowheelZoom(group, point, delta);
        }

        private void Domousemove(ContentControl con, MouseEventArgs e)
        {
            if (e.LeftButton != MouseButtonState.Pressed)
            {
                return;
            }
            var group = grid.FindResource("tfg") as TransformGroup;
            var transform = group.Children[1] as TranslateTransform;
            var position = e.GetPosition(con);
            transform.X -= mouseXY.X - position.X;
            transform.Y -= mouseXY.Y - position.Y;
            mouseXY = position;
        }

        private void DowheelZoom(TransformGroup group, Point point, double delta)
        {
            var pointToContent = group.Inverse.Transform(point);
            var scale = group.Children[0] as ScaleTransform;
            if (scale.ScaleX + delta < min) return;
            if (scale.ScaleY + delta > max) return;
            scale.ScaleX += delta;
            scale.ScaleY += delta;
            var translate = group.Children[1] as TranslateTransform;
            translate.X = -1 * ((pointToContent.X * scale.ScaleX) - point.X);
            translate.Y = -1 * ((pointToContent.Y * scale.ScaleY) - point.Y);
        }
    }
}
